# MIT License

# Copyright (c) 2016 Diogo Dutra <dutradda@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from swaggerit.utils import get_model_schema
from myreco.utils import ModuleClassLoader, get_items_model
from sqlalchemy.ext.declarative import AbstractConcreteBase, declared_attr
import sqlalchemy as sa
import ujson


class EnginesModelBase(AbstractConcreteBase):
    __tablename__ = 'engines'
    __schema__ = get_model_schema(__file__, 'base_swagger_schema.json')
    _jobs = dict()

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)
    name = sa.Column(sa.String(255), unique=True, nullable=False)
    configuration_json = sa.Column(sa.Text, nullable=False)

    @property
    def configuration(self):
        if not hasattr(self, '_configuration'):
            self._configuration = ujson.loads(self.configuration_json)
        return self._configuration

    @declared_attr
    def store_id(cls):
        return sa.Column(sa.ForeignKey('stores.id'), nullable=False)

    @declared_attr
    def core_id(cls):
        return sa.Column(sa.ForeignKey('engines_cores.id'), nullable=False)

    @declared_attr
    def item_type_id(cls):
        return sa.Column(sa.ForeignKey('items_types.id'), nullable=False)

    @declared_attr
    def core(cls):
        return sa.orm.relationship('EnginesCoresModel')

    @declared_attr
    def item_type(cls):
        return sa.orm.relationship('ItemsTypesModel')

    @declared_attr
    def store(cls):
        return sa.orm.relationship('StoresModel')

    @property
    def core_instance(self):
        if not hasattr(self, '_core_instance'):
            self._set_core_instance()

        return self._core_instance

    def _set_core_instance(self):
        core_class_ = self.get_core_class()
        self_dict = self._build_self_dict()
        items_model = get_items_model(self_dict)
        self._core_instance = core_class_(self_dict, items_model)

    def get_core_class(self):
        return ModuleClassLoader.load(self.core.configuration['core_module'])

    def _build_self_dict(self):
        todict_schema = {'variables': False}
        return self.todict(todict_schema)

    async def _setattr(self, attr_name, value, session, input_):
        if attr_name == 'configuration':
            value = ujson.dumps(value)
            attr_name = 'configuration_json'

        if attr_name == 'core_id':
            value = {'id': value}
            attr_name = 'core'

        if attr_name == 'item_type_id':
            value = {'id': value}
            attr_name = 'item_type'

        await super()._setattr(attr_name, value, session, input_)

    def _validate(self):
        if self.core is not None:
            self.core_instance.validate_config()

    def _format_output_json(self, dict_inst, schema):
        if schema.get('configuration') is not False:
            dict_inst.pop('configuration_json')
            dict_inst['configuration'] = self.configuration

        if schema.get('variables') is not False:
            dict_inst['variables'] = self.core_instance.get_variables()

    async def init(self, session, input_=None, **kwargs):
        store = kwargs.get('store')
        store_id = kwargs.get('store_id')

        if store is None and store_id is not None:
            store = await type(self).get_model('stores').get(session, {'id': store_id})

            if not store:
                raise SwaggerItModelError("Invalid store_id {}".format(store_id), instance=input_)
            else:
                kwargs['store'] = store[0]

        await super().init(session, input_=input_, **kwargs)



# MIT License

# Copyright (c) 2016 Diogo Dutra <dutradda@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from myreco.engines.cores.items_indices_map import ItemsIndicesMap
from myreco.engines.cores.filters.filters import BooleanFilterBy
from myreco.utils import build_item_key
from swaggerit.models.orm.jobs import JobsModel
from swaggerit.request import SwaggerRequest
from swaggerit.response import SwaggerResponse
from collections import defaultdict


class ItemsModelsCollection(JobsModel):
    __methods__ = defaultdict(dict)

    async def swagger_insert(self, req, session):
        items_model = self._get_model(req.query)
        if items_model is None:
            return SwaggerResponse(404)

        resp = await items_model.swagger_insert(req, session)
        await self._set_stock_filter(session, items_model)
        return resp

    def _get_model(self, query):
        store_id = query.pop('store_id')
        return self.get_model(build_item_key(self.__item_type__['name'], store_id))

    async def _set_stock_filter(self, session, items_model):
        items_indices_map = await ItemsIndicesMap(items_model).get_all(session)

        if items_indices_map.values():
            items_keys = set(await session.redis_bind.hkeys(items_model.__key__))
            items_indices_keys = set(items_indices_map.keys())
            remaining_keys = items_indices_keys.intersection(items_keys)
            old_keys = items_indices_keys.difference(items_keys)

            items = []
            self._set_stock_item(remaining_keys, items_model, items_indices_map, True, items)
            self._set_stock_item(old_keys, items_model, items_indices_map, False, items)

            stock_filter = BooleanFilterBy(items_model, 'stock')
            await stock_filter.update(session, items)

    def _set_stock_item(self, keys, items_model, items_indices_map, value, items):
        for key in keys:
            item = {}
            items_model.set_instance_ids(item, key)
            item.update({'stock': value, 'index': int(items_indices_map[key])})
            items.append(item)

    async def swagger_update_many(self, req, session):
        items_model = self._get_model(req.query)
        if items_model is None:
            return SwaggerResponse(404)

        resp = await items_model.swagger_update_many(req, session)
        await self._set_stock_filter(session, items_model)
        return resp

    def _update_path_params(self, items_models_collection, path_params):
        new_path_params = dict()
        id_names = items_models_collection.__item_type__['schema']['id_names']
        self.set_instance_ids(new_path_params, path_params['item_id'], keys=sorted(id_names))
        path_params.clear()
        path_params.update(new_path_params)

    async def swagger_get(self, req, session):
        items_model = self._get_model(req.query)
        if items_model is None:
            return SwaggerResponse(404)

        req = SwaggerRequest(
            req.url, req.method,
            path_params=list(req.path_params.values())[0],
            query=req.query,
            headers=req.headers,
            body=req.body,
            body_schema=req.body_schema,
            context=req.context
        )
        return await items_model.swagger_get(req, session)

    async def swagger_get_all(self, req, session):
        items_model = self._get_model(req.query)
        if items_model is None:
            return SwaggerResponse(404)

        return await items_model.swagger_get_all(req, session)


def build_items_models_collection_schema_base(base_uri, schema, patch_schema, id_names_uri):
    return {
        base_uri: {
            'parameters': [{
                'name': 'Authorization',
                'in': 'header',
                'required': True,
                'type': 'string'
            },{
                'name': 'store_id',
                'in': 'query',
                'required': True,
                'type': 'integer'
            }],
            'post': {
                'parameters': [{
                    'name': 'body',
                    'in': 'body',
                    'required': True,
                    'schema': {
                        'type': 'array',
                        'minItems': 1,
                        'items': schema
                    }
                }],
                'operationId': 'swagger_insert',
                'responses': {'201': {'description': 'Created'}}
            },
            'patch': {
                'parameters': [{
                    'name': 'body',
                    'in': 'body',
                    'required': True,
                    'schema': {
                        'type': 'array',
                        'minItems': 1,
                        'items': patch_schema
                    }
                }],
                'operationId': 'swagger_update_many',
                'responses': {'200': {'description': 'Updated'}}
            },
            'get': {
                'parameters': [{
                    'name': 'page',
                    'in': 'query',
                    'type': 'integer',
                    'default': 1
                },{
                    'name': 'items_per_page',
                    'in': 'query',
                    'type': 'integer',
                    'default': 1000
                },{
                    'name': 'item_key',
                    'in': 'query',
                    'type': 'array',
                    'items': {'type': 'string'}
                }],
                'operationId': 'swagger_get_all',
                'responses': {'200': {'description': 'Got'}}
            },
        },
        id_names_uri: {
            'parameters': [{
                'name': 'Authorization',
                'in': 'header',
                'required': True,
                'type': 'string'
            },{
                'name': 'store_id',
                'in': 'query',
                'required': True,
                'type': 'integer'
            },{
                'name': 'item_key',
                'in': 'path',
                'required': True,
                'type': 'string'
            }],
            'get': {
                'operationId': 'swagger_get',
                'responses': {'200': {'description': 'Got'}}
            }
        }
    }


# MIT License

# Copyright (c) 2016 Diogo Dutra <dutradda@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from myreco.items_types.items.model import ItemsModelBaseMeta
from myreco.items_types.items.models_collection import (
    ItemsModelsCollection, build_items_models_collection_schema_base)
from myreco.utils import build_item_key
from swaggerit.models.orm.factory import FactoryOrmModels
from swaggerit.utils import get_model_schema, get_dir_path
from swaggerit.response import SwaggerResponse
from swaggerit.method import SwaggerMethod
from sqlalchemy.ext.declarative import AbstractConcreteBase, declared_attr
from jsonschema import ValidationError, Draft4Validator
from copy import deepcopy
import sqlalchemy as sa
import ujson


class ItemsTypesModelBase(AbstractConcreteBase):
    __tablename__ = 'items_types'
    __schema__ = get_model_schema(__file__)
    __schema_dir__ = get_dir_path(__file__)
    __items_models_colletions__ = dict()

    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column(sa.String(255), unique=True, nullable=False)
    schema_json = sa.Column(sa.Text, nullable=False)

    @declared_attr
    def stores(cls):
        return sa.orm.relationship('StoresModel', uselist=True, secondary='items_types_stores')

    async def _setattr(self, attr_name, value, session, input_):
        if attr_name == 'schema':
            self._validate_input(value)
            value = ujson.dumps(value)
            attr_name = 'schema_json'

        await super()._setattr(attr_name, value, session, input_)

    def _validate_input(self, schema):
        Draft4Validator.check_schema(schema)

        for id_name in schema['id_names']:
            if id_name not in schema.get('properties', {}):
                raise ValidationError(
                    "id_name '{}' was not found in schema properties".format(id_name),
                    instance=schema['id_names'], schema=schema)

    def _format_output_json(self, dict_inst, todict_schema):
        if todict_schema.get('schema') is not False:
            if 'schema_json' in dict_inst:
                dict_inst['schema'] = ujson.loads(dict_inst.pop('schema_json'))

                schema_properties = dict_inst['schema'].get('properties', {})
                schema_properties_names = sorted(schema_properties.keys())
                dict_inst['available_filters'] = \
                    [{'name': name, 'schema': schema_properties[name]} \
                        for name in schema_properties_names]

    @classmethod
    async def build_all_items_models_collections(cls, session):
        items_types = await cls.get(session)
        return cls._build_items_models_collections(items_types)

    @classmethod
    def _build_items_models_collections(cls, items_types):
        return [cls._build_items_models_collection(item_type) for item_type in items_types]

    @classmethod
    def _build_items_models_collection(cls, item_type):
        name, schema = item_type['name'], item_type['schema']
        id_names = schema['id_names']
        class_name = cls._build_class_name(name) + 'Collection'
        key = build_item_key(name, 'collection')
        [cls._build_items_model(item_type, store) for store in item_type['stores']]
        item_model_key = key.replace('_collection', '')
        items_models_collection_class = cls._get_items_models_collection_class()
        items_models_collection_class.__key__ = key
        base_uri = '/{}'.format(item_model_key)
        items_models_collection_class.__schema__ = \
            cls._build_items_models_collection_schema(base_uri, schema, id_names)
        items_models_collection_class.__item_type__ = item_type
        items_models_collection_class.__item_type_model__ = cls
        items_models_collection = items_models_collection_class()
        cls.__items_models_colletions__[item_model_key] = items_models_collection

        if cls.__api__ is not None:
            cls._set_items_models_collection_methods(items_models_collection, base_uri)
            cls.__api__.update_swagger_paths(items_models_collection)

        return items_models_collection

    @classmethod
    def _build_class_name(cls, *names):
        final_name = ''
        for name in names:
            name = name.split(' ')
            for in_name in name:
                final_name += in_name.capitalize()

        return final_name + 'Model'

    @classmethod
    def _build_items_model(cls, item_type, store):
        class_name = cls._build_class_name(item_type['name'], store['name'])
        key = build_item_key(item_type['name'], store['id'])
        id_names = item_type['schema']['id_names']
        items_model = \
            FactoryOrmModels.make_redis(class_name, id_names, key, metaclass=ItemsModelBaseMeta)
        items_model.__item_type__ = item_type
        return items_model

    @classmethod
    def _get_items_models_collection_class(cls):
        return ItemsModelsCollection

    @classmethod
    def _build_items_models_collection_schema(cls, base_uri, schema, id_names):
        id_names_uri = base_uri + '/{item_key}'
        patch_schema = deepcopy(schema)
        required = patch_schema.get('required') # why it?
        if required:
            patch_schema['required'] = [req for req in required if req in id_names]
        properties = patch_schema.get('properties')
        if properties:
            properties['_operation'] = {'enum': ['delete', 'update']}

        swagger_schema = \
            build_items_models_collection_schema_base(base_uri, schema, patch_schema, id_names_uri)

        return swagger_schema

    @classmethod
    def _set_items_models_collection_methods(cls, items_models_collection, base_uri):
        for path, method, handler in cls.__api__.get_model_methods(items_models_collection):
            key = cls._get_items_model_colletion_method_key(path, (path == base_uri))
            items_models_collection.__methods__[key][method] = handler

    @classmethod
    def _get_items_model_colletion_method_key(cls, path, first_condition):
        if first_condition:
            key = 'base'
        elif path.endswith('import_data_file'):
            key = 'import_data_file'
        elif path.endswith('update_filters'):
            key = 'update_filters'
        else:
            key = 'id'

        return key

    @classmethod
    async def insert(cls, session, objs, commit=True, todict=True, **kwargs):
        objs = await type(cls).insert(cls, session, objs, commit=commit, todict=todict, **kwargs)
        cls._build_items_models_collections(objs)
        return objs

    @classmethod
    async def update(cls, session, objs, commit=True, todict=True, ids=None, **kwargs):
        old_items_types = await cls.get(session, ids=ids)
        objs = await type(cls).update(cls, session, objs, commit=commit, todict=todict, ids=ids, **kwargs)
        cls._rebuild_items_models_collections(old_items_types, objs)
        return objs

    @classmethod
    def _rebuild_items_models_collections(cls, old_items_types, new_items_types):
        new_names = [item_type['name'] for item_type in new_items_types]

        old_items_types_remove = \
            [item_type for item_type in old_items_types if item_type['name'] not in new_names]

        old_items_types_rebuild = \
            [item_type for item_type in old_items_types if item_type['name'] in new_names]

        old_names = [item_type['name']
            for item_type in old_items_types_remove+old_items_types_rebuild]

        new_items_types = \
            [item_type for item_type in new_items_types if item_type['name'] not in old_names]

        for item_type in old_items_types_remove:
            cls._remove_items_models_collection(item_type)

        for item_type in old_items_types_rebuild:
            cls._rebuild_items_models_collection(item_type)

        for item_type in new_items_types:
            cls._build_items_models_collection(item_type)

    @classmethod
    def _rebuild_items_models_collection(cls, item_type):
        cls._remove_items_models_collection(item_type)
        cls._build_items_models_collection(item_type)

    @classmethod
    def _remove_items_models_collection(cls, item_type):
        for store in item_type['stores']:
            key = build_item_key(item_type['name'], store['id'])
            type(cls).__all_models__.pop(key)

        model = type(cls).__all_models__.pop(build_item_key(item_type['name'], 'collection'))
        cls.__api__.remove_swagger_paths(model)

    @classmethod
    async def delete(cls, session, ids, commit=True, **kwargs):
        items_types = await cls.get(session, ids=ids)
        await type(cls).delete(cls, session, ids, commit=commit, **kwargs)
        [cls._remove_items_models_collection(item_type) for item_type in items_types]

    @classmethod
    async def items_models_handler(cls, req, session):
        items_model_name = req.path_params['items_model_name']
        items_models_collection = cls.__items_models_colletions__.get(items_model_name)

        if items_models_collection is None:
            methods = None
        else:
            key = cls._get_items_model_colletion_method_key(
                req.url, (not 'item_key' in req.path_params))
            methods = items_models_collection.__methods__.get(key)

        if methods is None:
            return SwaggerResponse(404)

        method = methods.get(req.method)

        if method is None:
            methods = [k.upper() for k in methods.keys()]
            return SwaggerResponse(405, headers={'Allow', ', '.join(methods)})

        return await method(req, session)


def build_items_types_stores_table(metadata, **kwargs):
    return sa.Table(
        'items_types_stores', metadata,
        sa.Column('item_type_id', sa.Integer, sa.ForeignKey('items_types.id', ondelete='CASCADE', onupdate='CASCADE'), primary_key=True),
        sa.Column('store_id', sa.Integer, sa.ForeignKey('stores.id', ondelete='CASCADE', onupdate='CASCADE'), primary_key=True),
        **kwargs)



# MIT License

# Copyright (c) 2016 Diogo Dutra <dutradda@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from myreco.items_types.import_data_file_model import ItemsTypesModelImportDataFileBase
from myreco.items_types.items.update_filters_models_collection import ItemsModelsCollectionFiltersUpdater


class ItemsTypesModelUpdateFiltersBase(ItemsTypesModelImportDataFileBase):

    @classmethod
    def _build_items_models_collection_schema(cls, key, schema, id_names):
        update_filters_uri = '{}/update_filters'.format(key)
        schema = ItemsTypesModelImportDataFileBase.\
            _build_items_models_collection_schema(key, schema, id_names)

        schema[update_filters_uri] = {
            'parameters': [{
                'name': 'Authorization',
                'in': 'header',
                'required': True,
                'type': 'string'
            },{
                'name': 'store_id',
                'in': 'query',
                'required': True,
                'type': 'integer'
            }],
            'post': {
                'operationId': 'post_update_filters_job',
                'responses': {'201': {'description': 'Executing'}}
            },
            'get': {
                'parameters': [{
                    'name': 'job_hash',
                    'in': 'query',
                    'type': 'string'
                }],
                'operationId': 'get_update_filters_job',
                'responses': {'200': {'description': 'Got'}}
            }
        }
        return schema

    @classmethod
    def _get_items_models_collection_class(cls):
        return ItemsModelsCollectionFiltersUpdater



# MIT License

# Copyright (c) 2016 Diogo Dutra <dutradda@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from myreco.items_types.base_model import ItemsTypesModelBase
from myreco.items_types.items.data_importer_models_collection import ItemsModelsCollectionDataImporter


class ItemsTypesModelImportDataFileBase(ItemsTypesModelBase):

    @classmethod
    def _build_items_models_collection_schema(self, key, schema, id_names):
        import_data_file_uri = '{}/import_data_file'.format(key)
        schema = ItemsTypesModelBase._build_items_models_collection_schema(key, schema, id_names)
        schema[import_data_file_uri] = {
                'parameters': [{
                    'name': 'Authorization',
                    'in': 'header',
                    'required': True,
                    'type': 'string'
                },{
                    'name': 'store_id',
                    'in': 'query',
                    'required': True,
                    'type': 'integer'
                },{
                    'name': 'upload_file',
                    'in': 'query',
                    'default': True,
                    'type': 'boolean'
                }],
                'post': {
                    'parameters': [{
                        'name': 'data_file',
                        'in': 'body',
                        'required': True,
                        'schema': {}
                    }],
                    'consumes': ['application/zip'],
                    'operationId': 'post_import_data_file_job',
                    'responses': {'200': {'description': 'Posted'}}
                },
                'get': {
                    'parameters': [{
                        'name': 'job_hash',
                        'in': 'query',
                        'type': 'string'
                    }],
                    'operationId': 'get_import_data_file_job',
                    'responses': {'200': {'description': 'Got'}}
                }
            }
        return schema

    @classmethod
    def _get_items_models_collection_class(self):
        return ItemsModelsCollectionDataImporter



# MIT License

# Copyright (c) 2016 Diogo Dutra <dutradda@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from swaggerit.utils import set_logger
from myreco.engines.cores.utils import build_engine_key_prefix
import zlib
import numpy as np


class RedisObjectBase(object):

    def __init__(self, engine_core):
        self._engine_core = engine_core
        self._set_redis_key()
        set_logger(self, self._redis_key)

    def _set_redis_key(self):
        self._redis_key = build_engine_key_prefix(self._engine_core.engine)

    def _pack_array(self, array):
        return zlib.compress(array.tobytes())

    def _unpack_array(self, array, dtype):
        if array is not None:
            return np.fromstring(zlib.decompress(array), dtype=dtype)
        else:
            return None
